;;; -*- Mode:Asm Mode:outline-minor-mode outline-regexp:";;;+" comment-start: "; " -*-
;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; equates-lpc1769.asm

;;; This contains equates that apply to all LPC17xx chips.  A separate
;;; include file will have board-specific equates and routines.
        
        ;; RAM on LPC17xx starts at 0x1000 0000 (16KB) or 0x1000 4000
        ;; (32KB) (i.e., the LPC1769 has both of those banks for a
        ;; total of 48KB).  There is an additional 16KB bank at 0x2008
        ;; 0000 in some family members, including the LPC1769.  Note
        ;; this latter bank is not contiguous with the other banks.

        ;; The clock starts off with the IRC (internal RC oscillator),
        ;; running at 4 MHz (1%).  By default, PCLK is CCLK/4.

        ;; P2.10 is the ISP enable pin (for boot loading)
        
        .equ CLKSRCSEL, 0x400FC10C
           ;; bits 1:0     00 = internal 4 MHZ RC,  01 = main oscillator,  10 = RTC,  11 = reserved
        
        ;; main PLL
        .equ PLL0CON,   0x400FC080
        .equ PLL0CFG,   0x400FC084
        .equ PLL0STAT,  0x400FC088
        .equ PLL0FEED,  0x400FC08C

        ;; USB PLL
        .equ PLL1CON,   0x400FC0A0
        .equ PLL1CFG,   0x400FC0A4
        .equ PLL1STAT,  0x400FC0A8
        .equ PLL1FEED,  0x400FC0AC
        
        ;; Clock dividers
        .equ CCLKCFG,   0x400FC104 ; CPU clock divider
          ;; set bits 7:0  to 1 less than the desired divider ( b'00000011' divides by 4)

        .equ USBCLKCFG, 0x400FC108

        .equ SCS,       0x400FC1A0
           ;;  3:0   reserved
           ;;    4   OSCRANGE       0 = main osc 1-20 MHz,   1 = main osc 15-25 MHz
           ;;    5   OSCEN          0 = main osc disabled,   1 = main osc enabled
           ;;    6   OSCSTAT        0 = main osc not ready,  1 = main osc ready

        ;; Peripheral Clock Selection Registers
        
        ;; for following two registers, the 2 bits for each PCLK set the divider as follows
        ;;    00  =  CCLK/4
        ;;    01  =  CCLK
        ;;    10  =  CCLK/2
        ;;    11  =  CCLK/8 (with some exceptions)
        ;; default is 00 (divide by 4)
        .equ PCLKSEL0,  0x400FC1A8
           ;; bits 3:2 PCLK_TIMER0
           ;; bits 5:4 PCLK_TIMER1
           ;; bits 7:6 PCLK_UART0
           ;; bits 9:8 PCLK_UART1
           ;;  etc
        
        .equ PCLKSEL1,  0x400FC1AC
           ;; bits 13:12 PCLK_TIMER2
           ;; bits 15:14 PCLK_TIMER3
           ;; bits 17:16 PCLK_UART2
           ;; bits 19:18 PCLK_UART3
           ;;  etc

        ;; Power control
        .equ PCON,      0x400FC0C0
        .equ PCONP,     0x400FC0C4
           .equ PCTIM0,    BIT1    ; on at reset
           .equ PCTIM1,    BIT2    ; on at reset
           .equ PCUART0,   BIT3    ; on at reset
           .equ PCUART1,   BIT4    ; on at reset
           .equ PCTIM2,    BIT22   ; off at reset
           .equ PCTIM3,    BIT23   ; off at reset
           .equ PCUART2,   BIT24   ; off at reset
           .equ PCUART3,   BIT25   ; off at reset
        
        ;; Utility
        .equ CLKOUTCFG, 0x400FC1C8

        ;; Flash
        .equ FLASHCFG, 0x400FC000 ; Flash Accelerator Configuration Register, reset value 0x303A
        
        ;; Interrupts
        .equ ISER0, 0xE000E100  ; Interrupt Set-Enable Register 0
        .equ ISER1, 0xE000E104  ; Interrupt Set-Enable Register 1
        .equ ICER0, 0xE000E180  ; Interrupt Clear-Enable Register 0
        .equ ICER1, 0xE000E184  ; Interrupt Clear-Enable Register 1
        .equ ICSP0, 0xE000E200  ; Interrupt Set-Pending Register 0
        .equ ICSP1, 0xE000E204  ; Interrupt Set-Pending Register 1
        .equ ICCP0, 0xE000E280  ; Interrupt Clear-Pending Register 0
        .equ ICCP1, 0xE000E284  ; Interrupt Clear-Pending Register 1
        .equ IABR0, 0xE000E300  ; Interrupt Active Bit Register 0
        .equ IABR1, 0xE000E304  ; Interrupt Active Bit Register 1
        .equ IPR0,  0xE000E400  ; Interrupt Priority Register 0
        .equ IPR1,  0xE000E404  ; Interrupt Priority Register 1
        .equ IPR2,  0xE000E408  ; Interrupt Priority Register 2
        .equ IPR3,  0xE000E40C  ; Interrupt Priority Register 3
        .equ IPR4,  0xE000E410  ; Interrupt Priority Register 4
        .equ IPR5,  0xE000E414  ; Interrupt Priority Register 5
        .equ IPR6,  0xE000E418  ; Interrupt Priority Register 6
        .equ IPR7,  0xE000E41C  ; Interrupt Priority Register 7
        .equ IPR8,  0xE000E420  ; Interrupt Priority Register 8
        .equ STIR,  0xE000EF00  ; Software Trigger Interrupt Register

        ;; Pin Selection Registers (see section 8.1, page 104 of User Manual)
        ;;  a pair of bits for each pin:
        ;;     00   primary function (typically GPIO) (default)
        ;;     01   first alternate function
        ;;     10   second alternate function
        ;;     11   third alternate function

        ;; Pin Mode registers
        ;;  the following 4 settings apply to the pin when in input mode (and half-apply
        ;;  when the pin is output open drain)
        ;;   00   on-chip pull-up enabled (default)
        ;;   01   repeater mode (prevents floating if input pin is temporarily not driven)
        ;;   10   no pull-up, no pull-down
        ;;   11   on-chip pull-down enabled
        ;; for PINMODE_ODx,   0 = output is push-pull,  1 = output is open drain

        ;; see user manual, section 8.5.1, page 108 for bits for alternate functions
        .equ PINSEL0,  0x4002C000
        .equ PINSEL1,  0x4002C004
        .equ PINSEL2,  0x4002C008
        .equ PINSEL3,  0x4002C00C
        .equ PINSEL4,  0x4002C010
        .equ PINSEL5,  0x4002C014
        .equ PINSEL6,  0x4002C018
        .equ PINSEL7,  0x4002C01C
        .equ PINSEL8,  0x4002C020
        .equ PINSEL9,  0x4002C024
        .equ PINSEL10, 0x4002C028 ; controls Trace function on pins P2.2 through P2.6

        .equ PINMODE0,  0x4002C040
        .equ PINMODE1,  0x4002C044
        .equ PINMODE2,  0x4002C048
        .equ PINMODE3,  0x4002C04C
        .equ PINMODE4,  0x4002C050
        .equ PINMODE5,  0x4002C054
        .equ PINMODE6,  0x4002C058
        .equ PINMODE7,  0x4002C05C
        .equ PINMODE8,  0x4002C060
        .equ PINMODE9,  0x4002C064

        .equ PINMODE_OD0,  0x4002C068
        .equ PINMODE_OD1,  0x4002C06C
        .equ PINMODE_OD2,  0x4002C070
        .equ PINMODE_OD3,  0x4002C074
        .equ PINMODE_OD4,  0x4002C078

        .equ I2CPADCFG,    0x4002C07C
        
        
        ;; GPIO

        .equ FIO0DIR,      0x2009C000 ; port direction, 0 (default) = input
        .equ FIO1DIR,      0x2009C020
        .equ FIO2DIR,      0x2009C040
        .equ FIO3DIR,      0x2009C060
        .equ FIO4DIR,      0x2009C080
        
        .equ FIO0MASK,     0x2009C010 ; port direction, 0 (default) = input
        .equ FIO1MASK,     0x2009C030
        .equ FIO2MASK,     0x2009C050
        .equ FIO3MASK,     0x2009C070
        .equ FIO4MASK,     0x2009C090

        .equ FIO0PIN,      0x2009C014
        .equ FIO1PIN,      0x2009C034
        .equ FIO2PIN,      0x2009C054
        .equ FIO3PIN,      0x2009C074
        .equ FIO4PIN,      0x2009C094
        
        .equ FIO0SET,      0x2009C018
        .equ FIO1SET,      0x2009C038
        .equ FIO2SET,      0x2009C058
        .equ FIO3SET,      0x2009C078
        .equ FIO4SET,      0x2009C098
        
        .equ FIO0CLR,      0x2009C01C
        .equ FIO1CLR,      0x2009C03C
        .equ FIO2CLR,      0x2009C05C
        .equ FIO3CLR,      0x2009C07C
        .equ FIO4CLR,      0x2009C09C

        .equ IO0IntEnR,    0x40028090
        .equ IO2IntEnR,    0x400280B0
        .equ IO0IntEnF,    0x40028094
        .equ IO2IntEnF,    0x400280B4
        .equ IO0IntStatR,  0x40028084
        .equ IO2IntStatR,  0x400280A4
        .equ IO0IntStatF,  0x40028088
        .equ IO2IntStatF,  0x400280A8
        .equ IO0IntClr,    0x4002808C
        .equ IO2IntClr,    0x400280AC
        .equ IOIntStatus,  0x40028080

        ;; UARTs see chapter 14 user manual, page 298

        .equ U0BASE,       0x4000C000
        .equ U1BASE,       0x40010000

            .equ URBR,   0x00      ; receive buffer register
            .equ UTHR,   0x00      ; transmit holding register
            .equ UDATA,  0x00      ; (same as above two)
            .equ UDLL,   0x00      ; divisor latch lsb
            .equ UDLM,   0x04      ; divisor latch msb
              ;; baud rate clock must be 16 x desired baud rate
              ;; baud rate clock =  PCLK divided by UDLM[7:0]:UDLL[7:0]

            .equ UFCR,   0x08      ; fifo control register
              ;; bit 0    enable (must be set!)
              ;; bit 1    Rx FIFO  write a 1 to clear receive FIFO
              ;; bit 2    Tx FIFO  write a 1 to clear transmit FIFO
              ;; bit 3    DMA mode
              ;; bit 5:4  reserved
              ;; bit 7:6  Rx trigger level (for interrupt or DMA)
              ;;           00 =  1 char
              ;;           01 =  4 char
              ;;           10 =  8 char
              ;;           11 = 14 char

            .equ ULCR,   0x0C      ; line control register
               ;; 1:0 word length        '11' for 8-bit
               ;;   2 stop bit select    '0' = 1 stop bit, '1' = 2  bits
               ;;   3 parity enable      '0' = no parity
               ;; 5:4 parity select      '00' = odd 
               ;;   6 break control      '0' = disable break transmission
               .equ DLAB, BIT7     ; divisor latch access bit

            .equ ULSR,   0x14      ; line status register
                                      ; bit 0 RDR (receive data ready)
                                      ; bit 5 THRE (transmit holding register empty)
            .equ USCR,   0x1C      ; scratch pad register      
            .equ UACR,   0x20
            .equ UICR,   0x24
            .equ UFDR,   0x28
                 ;; for fractional divider registers,
                 ;;   bits  7:4  MulVal  (must be >= 1), default is 1
                 ;;   bits  3:0  DivAddVal, 0 = disabled
 

        .equ U0RBR,        0x4000C000
        .equ U1RBR,        0x40010000
        .equ U2RBR,        0x40098000
        .equ U3RBR,        0x4009C000
        .equ U0THR,        0x4000C000
        .equ U1THR,        0x40010000
        .equ U2THR,        0x40098000
        .equ U3THR,        0x4009C000
        .equ U0DLL,        0x4000C000
        .equ U1DLL,        0x40010000
        .equ U2DLL,        0x40098000
        .equ U3DLL,        0x4009C000
        .equ U0DLM,        0x4000C004
        .equ U1DLM,        0x40010004
        .equ U2DLM,        0x40098004
        .equ U3DLM,        0x4009C004
        .equ U0IER,        0x4000C004
        .equ U1IER,        0x40010004
        .equ U2IER,        0x40098004
        .equ U3IER,        0x4009C004
        .equ U0IIR,        0x4000C008
        .equ U1IIR,        0x40010008
        .equ U2IIR,        0x40098008
        .equ U3IIR,        0x4009C008
        .equ U0FCR,        0x4000C008
        .equ U1FCR,        0x40010008
        .equ U2FCR,        0x40098008
        .equ U3FCR,        0x4009C008
        .equ U0LCR,        0x4000C00C
        .equ U1LCR,        0x4001000C
        .equ U2LCR,        0x4009800C
        .equ U3LCR,        0x4009C00C

        .equ U0LSR,        0x4000C014
        .equ U1LSR,        0x40010014
        .equ U2LSR,        0x40098014
        .equ U3LSR,        0x4009C014

        .equ U0SCR,        0x4000C01C
        .equ U1SCR,        0x4001001C
        .equ U2SCR,        0x4009801C
        .equ U3SCR,        0x4009C01C

        .equ U0ACR,        0x4000C020
        .equ U1ACR,        0x40010020
        .equ U2ACR,        0x40098020
        .equ U3ACR,        0x4009C020
        .equ U0ICR,        0x4000C024
        .equ U2ICR,        0x40098024
        .equ U3ICR,        0x4009C024
        .equ U0FDR,        0x4000C028
        .equ U1FDR,        0x40010028
        .equ U2FDR,        0x40098028
        .equ U3FDR,        0x4009C028
          ;; for fractional divider registers,
          ;;   bits  7:4  MulVal  (must be >= 1), default is 1
          ;;   bits  3:0  DivAddVal, 0 = disabled
        
        .equ U0TER,        0x4000C030
        .equ U1TER,        0x40010030
        .equ U2TER,        0x40098030
        .equ U3TER,        0x4009C030

        ;; UART1 has some registers that the other UARTs lack
        ;;  (and UART1 lacks the ICR register)
        .equ U1MCR,        0x40010010
        .equ U1LSR,        0x40010014
        .equ U1MSR,        0x40010018
        .equ U1RS485CTRL,  0x4001004C
        .equ U1ADRMATCH,   0x40010050
        .equ U1RS485DLY,   0x40010054

        ;; baud rate calculation examples

        ;;  BR = (/ PCLK  (* 16 (+ (* 256 DLM) DLL) (+ 1 (/ DivAddVal MulVal))))
        
        ;;  where PCLK is the peripheral clock (using 4 MHz internal oscillator and the
        ;;  default divide-by-4, PCLK would be 1 MHz).
        ;;  constraints:
        ;;    1 <= MulVal <= 15
        ;;    0 <= DivAddVal <= 14
        ;;    DivAddVal < MulVal

        ;; See user manual figure 46, page 314, for an algorithm
        ;; Example, with PCLK of 1 MHz and desired baud rate of 38400 bps
        ;;    DLest = (/ PCLK (* 16 BR))  =  (/ 1000000.0 (* 16 38400)) =  1.6276041666666667
        ;;  A.  since DLest is not an integer, FRest = 1.5
        ;;      DLest = Int(PCLK/(16 * BR * FRest)) = (/ 1000000.0 (* 16 38400 1.5)) 1.0850694444444444
        ;;      FRest = PCLK/(16 * BR * DLest)  =  (/ 1000000.0 (* 16 38400 1))      1.6276041666666667
        ;;      is   (1.1 < FRest < 1.9) ?
        ;;      yes, we got it on the first try  (otherwise, go back to A and pick a different FRest
        ;;       between 1.1 and 1.9)
        ;;      So, DivAddVal = table (FRest) = table (1.6278)  =  5
        ;;          MulVal    = table (FRest) = table (1.6278)  =  8
        ;;          DLM       = DLest [15:8]
        ;;          DLL       = DLest [7:0]
        ;;  Finally, plug in our results to the original formula to see how close we are:
        ;;  baud rate  =  (PCLK / (((16 * DLM) + DLL) * (1 + (DivAddVal / MulVal)) ) )
        ;;  baud rate (without correcting with fractional divider)  =  (/  PCLK (+ (* 16 DLM) DLL)))
        ;;    (/  1000000 (+ (* 16 DLM) DLL))  = (/  1000000 (+ (* 16 0) 1))  1000000
        ;;  now scale it by the fractional divider of 5/8
        ;;    (/  (* 5 1000000) 8) 625000

        
        ;; 38400 BPS with 4 MHz CCLK and 1 MHz PCLK
        ;;
        ;;  BR = (/ 1000000  (* 16 (+ (* 256 0) 1) (+ 1 (/ 5.0 8))))  38461.53846153846
        ;;  err =  (/ (- 38462 38400) 38400.0)  0.0016145833333333333  = 0.16% off
        ;;  ok, that looks better.
        ;;      So, U0DLM := 0
        ;;          U0DLL := 1
        ;;          DivAddVal := 5
        ;;          MulVal    := 8
        
        ;; 9600 BPS with 4 MHz CCLK and 1 MHz PCLK
        ;;  BR = (/ 1000000  (* 16 (+ (* 256 0) 4) (+ 1 (/ 5.0 8))))  9615.384615384615
        ;;  err =  (/ (- 9615 9600) 9600.0)  0.0015625  = 0.16% off
        ;;      So, U0DLM := 0
        ;;          U0DLL := 4
        ;;          DivAddVal := 5
        ;;          MulVal    := 8

        ;; 38400 BPS with 12 MHz CCLK and 3 MHz PCLK
        ;;  BR = (/ 3000000  (* 16 (+ (* 256 0) 3) (+ 1 (/ 5.0 8))))  38461.53846153846
        ;;  err =  (/ (- 38461 38400) 38400.0)  0.0015885  = 0.16% off
        ;;      So, U0DLM := 0
        ;;          U0DLL := 3
        ;;          DivAddVal := 5
        ;;          MulVal    := 8

; Calculations for above
;  DLest = PCLK /  (16 x BR)  =  (/ 3000000.0  (* 16 38400))    =  4.8828125
;  DLest is not an integer, so continue
;  FRest = 1.5
;  DLest = Int(  PCLK / (16 x BR x FRest))  =  (/ 3000000.0  (*  16  38400  1.5)) 3.2552083333333335 = 3
;  FRest = PCLK / (16 x BR x DLest)   =  (/  3000000.0 (* 16 38400 3)) 1.6276041666666667
;  ok, FRest is in the right range (>  1.1  PRest  1.9) so use the lookup table 
;   DIVADDVAL = table(FRest)  =  table (1.62)  =  5
;   MULVAL    = table(FRest)  =  table (1.62)  =  8
;   DLM       = DLest = upper8bits(3)  =  0
;   DLL       = DLest = lower8bits(3)  =  3


        ;; Timers
        .equ T0IR,    0x40004000
        .equ T1IR,    0x40008000
        .equ T2IR,    0x40090000
        .equ T3IR,    0x40094000

        .equ T0TCR,   0x40004004
        .equ T1TCR,   0x40008004
        .equ T2TCR,   0x40090004
        .equ T3TCR,   0x40094004
        
        .equ T0TC,    0x40004008
        .equ T1TC,    0x40008008
        .equ T2TC,    0x40090008
        .equ T3TC,    0x40094008
        
        .equ T0PR,    0x4000400C
        .equ T1PR,    0x4000800C
        .equ T2PR,    0x4009000C
        .equ T3PR,    0x4009400C
 
        .equ T0PC,    0x40004010
        .equ T1PC,    0x40008010
        .equ T2PC,    0x40090010
        .equ T3PC,    0x40094010

        .equ T0MCR,   0x40004014
        .equ T1MCR,   0x40008014
        .equ T2MCR,   0x40090014
        .equ T3MCR,   0x40094014

        .equ T0MR0,   0x40004018
        .equ T1MR0,   0x40008018
        .equ T2MR0,   0x40090018
        .equ T3MR0,   0x40094018

        .equ T0MR1,   0x4000401C
        .equ T1MR1,   0x4000801C
        .equ T2MR1,   0x4009001C
        .equ T3MR1,   0x4009401C

        .equ T0MR2,   0x40004020
        .equ T1MR2,   0x40008020
        .equ T2MR2,   0x40090020
        .equ T3MR2,   0x40094020

        .equ T0MR3,   0x40004024
        .equ T1MR3,   0x40008024
        .equ T2MR3,   0x40090024
        .equ T3MR3,   0x40094024

        .equ T0CCR,   0x40004028
        .equ T1CCR,   0x40008028
        .equ T2CCR,   0x40090028
        .equ T3CCR,   0x40094028
        
        .equ T0CR0,   0x4000402C
        .equ T1CR0,   0x4000802C
        .equ T2CR0,   0x4009002C
        .equ T3CR0,   0x4009402C

        .equ T0CR1,   0x40004030
        .equ T1CR1,   0x40008030
        .equ T2CR1,   0x40090030
        .equ T3CR1,   0x40094030

        .equ T0EMR,   0x4000403C
        .equ T1EMR,   0x4000803C
        .equ T2EMR,   0x4009003C
        .equ T3EMR,   0x4009403C

        .equ T0CTCR,  0x40004070
        .equ T1CTCR,  0x40008070
        .equ T2CTCR,  0x40090070
        .equ T3CTCR,  0x40094070
